searchData={"items":[{"type":"module","title":"GuardedStruct","doc":"The creation of this macro will allow you to build `Structs` that provide you with a number of important options, including the following:\n\n1. Validation\n2. Sanitizing\n3. Constructor\n4. It provides the capacity to operate in a nested style simultaneously.\n\nSuppose you are going to collect a number of pieces of information from the user, and before doing anything else, you are going to sanitize them.\nAfter that, you are going to validate each piece of data, and if there are no issues, you will either display it in a proper output or save it somewhere else.\nAll of the characteristics that are associated with this macro revolve around cleaning and validating the data.\n\nThe features that we list below are individually based on a particular strategy and requirement, but thankfully, they may be combined and mixed in any way that you see fit.\n\nIt bestows to you a significant amount of authority in this sphere.\nAfter the initial version of this macro was obtained from the source of the `typed_struct` library, many sections of it were rewritten, or new concepts were taken from libraries in Rust and Scala and added to this library in the form of Elixir base.\n\nThe initial version of this macro can be found in the `typed_struct` library. Its base is a syntax that is very easy to comprehend, especially for non-technical product managers, and highly straightforward.\n\nBefore explaining the copyright, I must point out that the primary library, which is `typed_struct`, is no longer supported for a long time, so please pay attention to the following copyright.","ref":"GuardedStruct.html"},{"type":"module","title":"Copyright - GuardedStruct","doc":"The code in this module is based on the 'typed_struct' library (https://github.com/ejpcmac/typed_struct),\nwhich is licensed under the MIT License.\n\nModifications and additions have been made to enhance its capabilities as part of the current project.\n\n**MIT License**\n\nAdding new Copyright (c) [2023] [Shahryar Tavakkoli at [Mishka Group](https://github.com/mishka-group)]\n\n**Note:** If the license changes during the support of this project, this file will always remain on MIT","ref":"GuardedStruct.html#module-copyright"},{"type":"macro","title":"GuardedStruct.guardedstruct/2","doc":"#","ref":"GuardedStruct.html#guardedstruct/2"},{"type":"macro","title":"Defines a guarded struct - GuardedStruct.guardedstruct/2","doc":"The beginning of the block consists of the introduction of a `Struct` with the `guardedstruct` macro, which is solely responsible for recording a series of information in order to create a struct, as well as all of the fields with the `field` macro, and if you need to create another struct within this struct (in actuality, a module child within another module), you must use the `sub_field` macro.\n\n**Note:** there is no restriction on the number of times you can call the `sub_field` macro or the field macro within the context of the `sub_field` macro.\n\n**Note:** Because `Stract` does not prioritize the display of keys depending on your requirements, you do not need to follow the priority of the fields and call them in order to utilize the app.\nImplement the program's logic, regardless of what it might be.\n\n**Note:** Because of different limitations, if you want to write a test, you must first place the module in which you built the struct outside of the test macro. Once the struct has been built, you may then test it by calling it within the test macro itself. The examples it provides can also be found in the testing done by this library itself.\n\n**Note:** this library is only supported on versions of `Elixir 1.15` and higher, as well as `OTP 26`, and that the manufacturer does not offer bug patches for problems that occur in older software versions.\n\n**Note:** All of this library's dependencies are optional; nonetheless, if you require their use in your program, you will need to include them. We provide further explanation on the topic in the area you're looking for.\n\n> Before continuing with the discussion about the library section and also offering practical examples in this field, it is important to understand that when you construct a struct in a module, after compilation in the runtime of the program, each module includes the following functional functions:\n\n1. The `builder()` function is actually an action function, and it requires you to provide it with information in the form of a `map`.\n\n2. The `enforce_keys()` function: this method returns the necessary keys of the first layer of the struct. However, if you want to display all of the keys of the nested struct, you will need to enter the `:all` input, which is not yet implemented in this version.\n\n3. The `keys()` function has the same requirements as the `enforce_keys()` function, with the exception that it returns all of the keys, including the ones that aren't necessary.\n\n---\n\n**And also, any data that enters the `builder` function must go through the following path:**\n\n1. If the `map` currently uses the `string` data type, it will be converted to the `atom` data type.\n\n2. Eliminates the keys from the `struct` that are not present in the list\n\n3. Determines whether or not all of the essential keys have been transmitted.\n\n4. If you write your own custom validation, each field's validations will be checked.\n\n> It is important to notice that regardless of the circumstances, this macro also inspects the module itself. If there is a `validator` function but none of the functions are set, it calls the validator function directly from the module itself into the field itself.\n\n5. The output of the complete `struct` is entered into the mother validation, and the programmer is given the opportunity to write for the final output in this validation. This validation also provides the possibility of writing for the output of the struct.\n\n> This macro will call the struct's `main_validator` directly from the module it has been placed in if, in this section, the `main_validator`  is not set in the struct but is found in the module that contains the struct.\n\n6. If there were no problems in the previous phases (it is important to note that options 4 and 5 are not required), it will proceed to the next level of the program, which is the validation and custom Sanitizer stage.\n\n7. To begin, the Sanitizer will alter the data so that it corresponds to what you have called in each field, and it will not return any errors. Even if the Sanitizer programmer is not utilized in the required type as a result of an accidental oversight, the data will still be passed to the following stage.\n\n8. At this point, it will return an error or data for each field, depending on the validations that you called.\n\n9. At the end of the process, you will receive a tuple that will either have problems in it or the final data with an ok status.\n\n> It is important to keep in mind that if your `struct` is nested, all of the internal errors of these structs are also included in the list of problems. Additionally, the data will be sent to you when the status is positive, but only if you have called the parent of this struct.\n\n> Note that each nested struct can be used on its own and possesses all of the capabilities that have been discussed thus far. For instance, if you have module `A` and you utilized the `sub_field` that is named `auth` in it, you may now use it separately from the `A.Auth` Use. Use.\n\n---\n\n#","ref":"GuardedStruct.html#guardedstruct/2-defines-a-guarded-struct"},{"type":"macro","title":"Examples - GuardedStruct.guardedstruct/2","doc":"1. ##","ref":"GuardedStruct.html#guardedstruct/2-examples"},{"type":"macro","title":"Defining a struct layer without additional options - GuardedStruct.guardedstruct/2","doc":"```elixir\ndefmodule MyStruct do\n  use GuardedStruct\n\n  guardedstruct do\n    field :field_one, String.t()\n    field :field_two, integer(), enforce: true\n    field :field_three, boolean(), enforce: true\n    field :field_four, atom(), default: :hey\n  end\nend\n```\n\n---\n\n2. ##","ref":"GuardedStruct.html#guardedstruct/2-defining-a-struct-layer-without-additional-options"},{"type":"macro","title":"Define a struct with settings related to essential keys or `opaque` type - GuardedStruct.guardedstruct/2","doc":"###","ref":"GuardedStruct.html#guardedstruct/2-define-a-struct-with-settings-related-to-essential-keys-or-opaque-type"},{"type":"macro","title":"Options - GuardedStruct.guardedstruct/2","doc":"* `enforce` - if set to true, sets `enforce: true` to all fields by default.\nThis can be overridden by setting `enforce: false` or a default value on\nindividual fields.\n* `opaque` - if set to true, creates an opaque type for the struct.\n* `module` - if set, creates the struct in a submodule named `module`.\n\n```elixir\ndefmodule MyModule do\n  use GuardedStruct\n\n  guardedstruct enforce: true do\n    field(:enforced_by_default, term())\n    field(:not_enforced, term(), enforce: false)\n    field(:with_default, integer(), default: 1)\n    field(:with_false_default, boolean(), default: false)\n    field(:with_nil_default, term(), default: nil)\n  end\nend\n\n# OR opaque\n\ndefmodule MyModule do\n  use GuardedStruct\n\n  guardedstruct opaque: true do\n    field(:enforced_by_default, term())\n    field(:not_enforced, term(), enforce: false)\n    field(:with_default, integer(), default: 1)\n    field(:with_false_default, boolean(), default: false)\n    field(:with_nil_default, term(), default: nil)\n  end\nend\n\n# OR opaque\n\ndefmodule MyModule do\n  use GuardedStruct\n\n  guardedstruct do\n    field(:enforced_by_default, term())\n    field(:not_enforced, term(), enforce: true)\n    field(:with_default, integer(), default: 1)\n    field(:with_false_default, boolean(), default: false)\n    field(:with_nil_default, term(), default: nil)\n  end\nend\n\n# OR create sub module\n\ndefmodule TestModule do\n  use GuardedStruct\n\n  guardedstruct module: Struct do\n    field(:field, term())\n  end\nend\n```\n\n---\n\n3. ##","ref":"GuardedStruct.html#guardedstruct/2-options"},{"type":"macro","title":"Defining the struct by calling the validation module or calling from the module that contains the struct - GuardedStruct.guardedstruct/2","doc":"###","ref":"GuardedStruct.html#guardedstruct/2-defining-the-struct-by-calling-the-validation-module-or-calling-from-the-module-that-contains-the-struct"},{"type":"macro","title":"Options - GuardedStruct.guardedstruct/2","doc":"* `validator` - if set as tuple like this {ModuleName, :function_name} for each field,\nin fact you have a `builder` function that check the validation.\n\n```elixir\n# First, it looks at whether a validator has been set for each field,\n# otherwise it looks inside the module.\ndefmodule MyModule do\n  alias MyModule.AnotherModule\n  use GuardedStruct\n\n  guardedstruct do\n    field(:name, String.t(), validator: {AnotherModule, :validator})\n    field(:title, String.t())\n  end\n\n  def validator(:title, value) do\n    {:ok, :title, value}\n  end\n\n  # You can not use it, but it is mentioned here for test clarity\n  def validator(name, value) do\n    {:ok, name, value}\n  end\nend\n```\n\n- Output without error: `{:ok, :field_name, value}`\n- Output with error: `{:error, :field_name, ERROR MESSAGE}`\n\n---\n\n4. ##","ref":"GuardedStruct.html#guardedstruct/2-options"},{"type":"macro","title":"Define the struct by calling the `main_validator` for full access on the output - GuardedStruct.guardedstruct/2","doc":"###","ref":"GuardedStruct.html#guardedstruct/2-define-the-struct-by-calling-the-main_validator-for-full-access-on-the-output"},{"type":"macro","title":"Options - GuardedStruct.guardedstruct/2","doc":"* `main_validator` - if set as tuple like this {ModuleName, :function_name},\nfor guardedstruct, in fact you have a global validation.\n\n```elixir\n# First, it looks at whether a main_validator has been set for each field,\n# otherwise it looks inside the module.\ndefmodule MyModule do\n  alias MyModule.AnotherModule\n  use GuardedStruct\n\n  guardedstruct main_validator: {AnotherModule, :main_validator} do\n  field(:name, String.t())\n  field(:title, String.t())\n  end\n\n  # if `guardedstruct` has no `main_validator` which is configed\n  def main_validator(value) do\n    {:ok, value}\n  end\nend\n```\n\n- Output without error: `{:ok, value}`\n- Output with error: `{:error, :generalـreason, errors_list}`\n\n---\n\n5. ##","ref":"GuardedStruct.html#guardedstruct/2-options"},{"type":"macro","title":"Define struct with `derive` - GuardedStruct.guardedstruct/2","doc":"> derive is divided into two parts: `validate` and `sanitize`, which is priority with `sanitize`\n\n**It should be noted that in the following tables you can see that in order to use some derives, you need to add its dependency on your project.**\n\n\n##","ref":"GuardedStruct.html#guardedstruct/2-define-struct-with-derive"},{"type":"macro","title":"Sanitize - GuardedStruct.guardedstruct/2","doc":"| How to use | Dependencies | Description |\n| ---------- | ------------ | ----------- |\n| `\"sanitize(trim)\"` | NO | Trim your string |\n| `\"sanitize(upcase)\"` | NO | Upcase your string |\n| `\"sanitize(downcase)\"` | NO | Downcase your string |\n| `\"sanitize(capitalize)\"` | NO | Capitalize your string |\n| `\"sanitize(basic_html)\"` | `:html_sanitize_ex` | Sanitize your string base on `basic_html` |\n| `\"sanitize(html5)\"` | `:html_sanitize_ex` | Sanitize your string base on `html5` |\n| `\"sanitize(markdown_html)\"` | `:html_sanitize_ex` | Sanitize your string base on `markdown_html` |\n| `\"sanitize(strip_tags)\"` | `:html_sanitize_ex` | Sanitize your string base on `strip_tags` |\n| `\"sanitize(tag)\"` | `:html_sanitize_ex` | Sanitize your string base on `html_sanitize_ex` selection |\n\n##","ref":"GuardedStruct.html#guardedstruct/2-sanitize"},{"type":"macro","title":"Validate - GuardedStruct.guardedstruct/2","doc":"| How to use | Dependencies | Description |\n| ---------- | ------------ | ----------- |\n| `\"validate(string)\"` | NO | Validate if the data is string|\n| `\"validate(integer)\"` | NO | Validate if the data is integer|\n| `\"validate(list)\"` | NO | Validate if the data is list|\n| `\"validate(atom)\"` | NO | Validate if the data is atom|\n| `\"validate(bitstring)\"` | NO | Validate if the data is bitstring|\n| `\"validate(boolean)\"` | NO | Validate if the data is boolean|\n| `\"validate(exception)\"` | NO | Validate if the data is exception|\n| `\"validate(float)\"` | NO | Validate if the data is float|\n| `\"validate(function)\"` | NO | Validate if the data is function|\n| `\"validate(map)\"` | NO | Validate if the data is map|\n| `\"validate(nil_value)\"` | NO | Validate if the data is nil value|\n| `\"validate(not_nil_value)\"` | NO | Validate if the data is not nil value|\n| `\"validate(number)\"` | NO | Validate if the data is number|\n| `\"validate(pid)\"` | NO | Validate if the data is Elixir pid|\n| `\"validate(port)\"` | NO | Validate if the data is Elixir port|\n| `\"validate(reference)\"` | NO | Validate if the data is Elixir reference|\n| `\"validate(struct)\"` | NO | Validate if the data is struct|\n| `\"validate(tuple)\"` | NO | Validate if the data is tuple|\n| `\"validate(not_empty)\"` | NO | Validate if the data is not empty - binary, map, list|\n| `\"validate(max_len=10)\"` | NO | Validate if the data is more than 10 - Range, integer, binary|\n| `\"validate(min_len=10)\"` | NO | Validate if the data is less than 10 - Range, integer, binary|\n| `\"validate(url)\"` | NO | Validate if the data is url|\n| `\"validate(geo_url)\"` | `ex_url` | Validate if the data is geo url|\n| `\"validate(tell)\"` | `ex_url` | Validate if the data is tell|\n| `\"validate(tell=98)\"` | `ex_url` | Validate if the data is tell with country code|\n| `\"validate(email)\"` | `email_checker` | Validate if the data is email|\n| `\"validate(location)\"` | `ex_url` | Validate if the data is location|\n| `\"validate(string_boolean)\"` | NO | Validate if the data is string boolean|\n| `\"validate(datetime)\"` | NO | Validate if the data is datetime|\n| `\"validate(range)\"` | NO | Validate if the data is datetime|\n| `\"validate(date)\"` | NO | Validate if the data is datetime|\n| `\"validate(regex='^[a-zA-Z]+@mishka.group$')\"` | NO | Validate if the data is match with regex|\n| `\"validate(ipv4)\"` | NO | Validate if the data is ipv4|\n| `\"validate(not_empty_string)\"` | NO | Validate if the data is not empty string|\n| `\"validate(uuid)\"` | NO | Validate if the data is uuid|\n| `\"validate(enum=String[admin::user::banned])\"` | NO | Validate if the data is one of the enum value, which is String|\n| `\"validate(enum=Atom[admin::user::banned])\"` | NO | Validate if the data is one of the enum value, which is Atom|\n| `\"validate(enum=Integer[1::2::3])\"` | NO | Validate if the data is one of the enum value, which is Integer|\n| `\"validate(enum=Float[1.5::2.0::4.5])\"` | NO | Validate if the data is one of the enum value, which is Float|\n| `\"validate(enum=Map[%{status: 1}::%{status: 2}::%{status: 3}])\"` | NO | Validate if the data is one of the enum value, which is Map|\n| `\"validate(enum=Tuple[{:admin, 1}::{:user, 2}::{:banned, 3}])\"` | NO | Validate if the data is one of the enum value, which is Tuple|\n\n```elixir\ndefmodule MyModule do\n  use GuardedStruct\n\n  guardedstruct do\n    field(:id, integer(), derive: \"sanitize(trim) validate(integer, max_len=20, min_len=5)\")\n    field(:title, String.t(), derive: \"sanitize(trim, upcase) validate(not_empty_string)\")\n    field(:name, String.t(), derive: \"sanitize(trim, capitalize) validate(string, not_empty, max_len=20)\")\n  end\nend\n```\n\n---\n\n6. ##","ref":"GuardedStruct.html#guardedstruct/2-validate"},{"type":"macro","title":"Extending `derive` section - GuardedStruct.guardedstruct/2","doc":"###","ref":"GuardedStruct.html#guardedstruct/2-extending-derive-section"},{"type":"macro","title":"Options - GuardedStruct.guardedstruct/2","doc":"* `validate_derive` - It can be just one module or a list of modules\n* `sanitize_derive` - It can be just one module or a list of modules\n\n```elixir\ndefmodule TestValidate do\n  def validate(:testv1, input, field) do\n    if is_binary(input),\n      do: input,\n      else: {:error, field, :testv1, \"The name field must not be empty\"}\n  end\nend\n\ndefmodule TestValidate2 do\n  def validate(:testv2, input, field) do\n    if is_binary(input),\n      do: input,\n      else: {:error, field, :testv1, \"The name field must not be empty\"}\n  end\nend\n\ndefmodule TestSanitize do\n  def sanitize(:capitalize_v1, input) do\n    if is_binary(input), do: String.capitalize(input), else: input\n  end\nend\n\ndefmodule TestSanitize2 do\n  def sanitize(:capitalize_v2, input) do\n    if is_binary(input), do: String.capitalize(input), else: input\n  end\nend\n\ndefmodule MyModule do\n  use GuardedStruct\n\n  guardedstruct validate_derive: TestValidate, sanitize_derive: TestSanitize do\n    field(:id, integer(), derive: \"sanitize(trim) validate(not_exist)\")\n    field(:title, String.t(), derive: \"sanitize(trim) validate(string)\")\n    field(:name, String.t(), derive: \"sanitize(capitalize_v2) validate(string)\")\n  end\nend\n\n# OR you can extend with list of modules\n\ndefmodule MyModule do\n  use GuardedStruct\n\n  guardedstruct validate_derive: [TestValidate, TestValidate2], sanitize_derive: [TestSanitize, TestSanitize2] do\n    field(:id, integer(), derive: \"validate(ineteger)\")\n    field(:title, String.t(), derive: \"sanitize(trim) validate(string)\")\n    field(:name, String.t(), derive: \"sanitize(capitalize_v2) validate(string)\")\n  end\nend\n```\n---\n\n7. ##","ref":"GuardedStruct.html#guardedstruct/2-options"},{"type":"macro","title":"Struct definition with `validator` and `derive` simultaneously - GuardedStruct.guardedstruct/2","doc":"```elixir\n# In this code, name field has not custom validator module and function\n# Then it see the caller module for it\ndefmodule MyModule do\n  use GuardedStruct\n\n  guardedstruct do\n    field(:name, String.t(),\n      enforce: true,\n      derive: \"sanitize(trim, upcase) validate(not_empty)\"\n    )\n\n    field(:title, String.t(), derive: \"sanitize(trim, capitalize) validate(not_empty)\")\n  end\n\n  def validator(:name, value) do\n    if is_binary(value), do: {:ok, :name, \"Mishka   \"}, else: {:error, :name, \"No, never\"}\n  end\n\n  def validator(name, value) do\n    {:ok, name, value}\n  end\nend\n\n# OR with custom validator\n\ndefmodule MyModule do\n  alias MyModule.AnotherModule\n  use GuardedStruct\n\n  guardedstruct do\n    field(:name, String.t(), enforce: true, derive: \"sanitize(trim, capitalize) validate(not_empty)\" validator: {AnotherModule, :validator})\n    field(:title, String.t(), derive: \"sanitize(trim, capitalize) validate(not_empty)\")\n  end\n\n  # You can not use it, but it is mentioned here for test clarity\n  def validator(name, value) do\n    {:ok, name, value}\n  end\nend\n```\n---\n\n8. ##","ref":"GuardedStruct.html#guardedstruct/2-struct-definition-with-validator-and-derive-simultaneously"},{"type":"macro","title":"Define a nested and complex struct - GuardedStruct.guardedstruct/2","doc":"```elixir\ndefmodule TestNestedStruct do\n  use GuardedStruct\n\n  guardedstruct do\n    field(:name, String.t(),\n      derive:\n        \"sanitize(strip_tags, trim, capitalize) validate(string, not_empty, max_len=20, min_len=3)\"\n    )\n\n    field(:family, String.t(),\n      derive:\n        \"sanitize(basic_html, trim, capitalize) validate(string, not_empty, max_len=20, min_len=3)\"\n    )\n\n    field(:age, integer(), enforce: true, derive: \"validate(integer, max_len=110, min_len=18)\")\n\n    sub_field(:auth, struct(), enforce: true) do\n      field(:server, String.t(), derive: \"validate(regex='^[a-zA-Z]+@mishka.group$')\")\n\n      field(:identity_provider, String.t(),\n        derive: \"sanitize(strip_tags, trim, lowercase) validate(not_empty)\"\n      )\n\n      sub_field(:role, struct(), enforce: true) do\n        field(:name, String.t(),\n          derive:\n            \"sanitize(strip_tags, trim, lowercase) validate(enum=Atom[admin::user::banned])\"\n        )\n\n        field(:action, String.t(), derive: \"validate(string_boolean)\")\n\n        field(:status, String.t(),\n          derive: \"validate(enum=Map[%{status: 1}::%{status: 2}::%{status: 3}])\"\n        )\n      end\n\n      field(:last_activity, String.t(), derive: \"sanitize(strip_tags, trim) validate(datetime)\")\n    end\n\n    sub_field(:profile, struct()) do\n      field(:site, String.t(), derive: \"validate(url)\")\n\n      field(:nickname, String.t(), validator: {TestNestedStruct, :validator})\n    end\n\n    field(:username, String.t(),\n      enforce: true,\n      derive: \"sanitize(tag=strip_tags) validate(not_empty, max_len=20, min_len=3)\"\n    )\n  end\n\n  def validator(:nickname, value) do\n    if is_binary(value),\n      do: {:ok, :nickname, value},\n      else: {:error, :nickname, \"Invalid nickname\"}\n  end\n\n  def validator(field, value) do\n    {:ok, field, value}\n  end\nend\n```\n\n9. ##","ref":"GuardedStruct.html#guardedstruct/2-define-a-nested-and-complex-struct"},{"type":"macro","title":"Error and data output sample - GuardedStruct.guardedstruct/2","doc":"```elixir\n# Error\n{:error, :bad_parameters,\n  [\n    %{\n      field: :profile,\n      errors: {:bad_parameters, [%{message: \"Invalid nickname\", field: :nickname}]}\n    },\n    %{\n      field: :auth,\n      errors:\n        {:bad_parameters,\n        [\n          %{message: _msg, field: :last_activity, action: :datetime},\n          %{\n            field: :role,\n            errors:\n              {:bad_parameters,\n                [\n                  %{message: _msg1, field: :action, action: :string_boolean}\n                ]}\n          }\n        ]}\n    }\n  ]}\n\n# Data\n\n{:ok,\n  %MishkaDeveloperToolsTest.GuardedStructTest.TestNestedStruct{\n    username: \"mishka\",\n    profile: %MishkaDeveloperToolsTest.GuardedStructTest.TestNestedStruct.Profile{\n      nickname: \"mishka\",\n      site: \"https://elixir-lang.org\"\n    },\n    auth: %MishkaDeveloperToolsTest.GuardedStructTest.TestNestedStruct.Auth{\n      last_activity: \"2023-08-20 16:54:07.841434Z\",\n      role: %MishkaDeveloperToolsTest.GuardedStructTest.TestNestedStruct.Auth.Role{\n        action: \"true\",\n        name: :user,\n        status: %{status: 2}\n      },\n      identity_provider: \"google\",\n      server: \"users@mishka.group\"\n    },\n    age: 18,\n    family: \"Group\",\n    name: \"Mishka\"\n  }}\n```","ref":"GuardedStruct.html#guardedstruct/2-error-and-data-output-sample"},{"type":"module","title":"MishkaDeveloperTools","doc":"# Mishka Elixir Developer Tools\n\nWe tried to deliver a series of our client's [**CMS**](https://github.com/mishka-group/mishka-cms) built on [**Elixir**](https://elixir-lang.org/) at the start of the [**Mishka Group**](https://github.com/mishka-group) project, but we recently archived this open-source project and have yet to make plans to rework and expand it. This system was created using [**Phoenix**](https://www.phoenixframework.org/) and [**Phoenix LiveView**](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html). After a long period, a series of macros and functional modules emerged from this project and our other projects, which we are gradually publishing in this library.\n\n---\n\n- #### [GuardedStruct](https://github.com/mishka-group/mishka_developer_tools/blob/master/guidance/guarded-struct.md)\n\n- #### [Basic CRUD](https://github.com/mishka-group/mishka_developer_tools/blob/master/guidance/crud.md)\n\n---\n\n> Mishka developer tools provides some macros and modules to make creating your elixir application as easy as possible","ref":"MishkaDeveloperTools.html"},{"type":"module","title":"Installation - MishkaDeveloperTools","doc":"The package can be installed by adding `mishka_developer_tools` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mishka_developer_tools, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThe docs can be found at [https://hexdocs.pm/mishka_developer_tools](https://hexdocs.pm/mishka_developer_tools).","ref":"MishkaDeveloperTools.html#module-installation"},{"type":"behaviour","title":"MishkaDeveloperTools.DB.CRUD","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html"},{"type":"behaviour","title":"Simplified CRUD macro using Ecto - MishkaDeveloperTools.DB.CRUD","doc":"With this module, you can easily implement CRUD-related items in your file wherever you need to build a query.\nThese modules and their sub-macros were created more to create a one-piece structure, and you can implement your own custom items in umbrella projects.\nIn the first step, to use the following macros, you must bind the requested information in the relevant module that you have already created as follows.\n```elixir\nuse MishkaDeveloperTools.DB.CRUD,\n    module: YOURschemaMODULE,\n    repo: Your.Repo,\n    id: :uuid OR ANY_TYPE_YOU_WANT\n```\nIt should be noted that the following three parameters must be sent and also make sure you are connected to the database.\n\n```elixir\nmodule\nrepo\nid\n```","ref":"MishkaDeveloperTools.DB.CRUD.html#module-simplified-crud-macro-using-ecto"},{"type":"callback","title":"MishkaDeveloperTools.DB.CRUD.create/1","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#c:create/1"},{"type":"macro","title":"MishkaDeveloperTools.DB.CRUD.crud_add/1","doc":"#","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_add/1"},{"type":"macro","title":"Creating a record macro - MishkaDeveloperTools.DB.CRUD.crud_add/1","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_add/1-creating-a-record-macro"},{"type":"macro","title":"Example - MishkaDeveloperTools.DB.CRUD.crud_add/1","doc":"```elixir\ncrud_add(map_of_info like: %{\"name\" => \"Mishka\"})\n```\nThe input of this macro is a map and its output are a map. For example\n\n```elixir\n{:error, :add, repo_error()}\n{:ok, :add, repo_data()}\n```\n\nIf you want only the selected parameters to be separated from the list of submitted parameters and sent to the database, use the same macro with input 2\n\n###  Example\n```elixir\ncrud_add(map_of_info like: %{\"name\" => \"Mishka\"}, [\"name\"])\n```","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_add/1-example"},{"type":"macro","title":"MishkaDeveloperTools.DB.CRUD.crud_add/2","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_add/2"},{"type":"macro","title":"MishkaDeveloperTools.DB.CRUD.crud_delete/1","doc":"#","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_delete/1"},{"type":"macro","title":"delete a record from the database with the help of ID Macro - MishkaDeveloperTools.DB.CRUD.crud_delete/1","doc":"With the help of this macro, you can delete your requested record from the database.\nThe input of this macro is a UUID and its output is a map","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_delete/1-delete-a-record-from-the-database-with-the-help-of-id-macro"},{"type":"macro","title":"Example - MishkaDeveloperTools.DB.CRUD.crud_delete/1","doc":"```elixir\ncrud_delete(\"6d80d5f4-781b-4fa8-9796-1821804de6ba\")\ncrud_delete(\"6d80d5f4-781b-4fa8-9796-1821804de6ba\", [:comment, :post])\n```\nOutput:\nYou should note that this macro prevents the orphan data of the record requested to be deleted. So, use this macro when the other data is not dependent on the data with the ID sent by you.\n\nOutputs:\n\n```elixir\n{:error, :delete, repo_error()}\n{:error, :delete, {:error, :uuid | :not_found | :force_constraint, String.t()}}\n{:ok, :delete, repo_data()}\n```","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_delete/1-example"},{"type":"macro","title":"MishkaDeveloperTools.DB.CRUD.crud_delete/2","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_delete/2"},{"type":"macro","title":"MishkaDeveloperTools.DB.CRUD.crud_edit/1","doc":"#","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_edit/1"},{"type":"macro","title":"Edit a record in a database Macro - MishkaDeveloperTools.DB.CRUD.crud_edit/1","doc":"With the help of this macro, you can edit a record in the database with its ID. For this purpose, you must send the requested record ID along with the new Map parameters. Otherwise the macro returns the ID error.","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_edit/1-edit-a-record-in-a-database-macro"},{"type":"macro","title":"Example - MishkaDeveloperTools.DB.CRUD.crud_edit/1","doc":"```elixir\ncrud_edit(map_of_info like: %{\"id\" => \"6d80d5f4-781b-4fa8-9796-1821804de6ba\", \"name\" => \"Mishka\"})\n```\n> Note that the sending ID must be of UUID type.\n\nThe input of this macro is a map and its output are a map. For example\n\n```elixir\n{:error, :edit, repo_error()}\n{:ok, :edit, repo_data()}\n{:error, :edit, {:error, :uuid | :not_found, String.t()}}\n```\n\nIt should be noted that if you want only the selected fields to be separated from the submitted parameters and sent to the database, use the macro with dual input.","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_edit/1-example"},{"type":"macro","title":"Example - MishkaDeveloperTools.DB.CRUD.crud_edit/1","doc":"```elixir\ncrud_edit(map_of_info like: %{\"id\" => \"6d80d5f4-781b-4fa8-9796-1821804de6ba\", \"name\" => \"Mishka\"}, , [\"id\", \"name\"])\n```","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_edit/1-example"},{"type":"macro","title":"MishkaDeveloperTools.DB.CRUD.crud_edit/2","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_edit/2"},{"type":"macro","title":"MishkaDeveloperTools.DB.CRUD.crud_get_by_field/2","doc":"#","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_get_by_field/2"},{"type":"macro","title":"Macro Find a record in the database with the help of the requested field - MishkaDeveloperTools.DB.CRUD.crud_get_by_field/2","doc":"With the help of this macro, you can find a field with the value you want, if it exists in the database. It should be noted that the field name must be entered as a String.\n\n\n# Example\n```elixir\ncrud_get_by_field(\"email\", \"info@trangell.com\")\n```\n\nOutputs:\n\n```\n{:error, :not_found, String.t()} | struct()\n```","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_get_by_field/2-macro-find-a-record-in-the-database-with-the-help-of-the-requested-field"},{"type":"macro","title":"MishkaDeveloperTools.DB.CRUD.crud_get_record/1","doc":"#","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_get_record/1"},{"type":"macro","title":"Macro Finding a record in a database with the help of ID - MishkaDeveloperTools.DB.CRUD.crud_get_record/1","doc":"With the help of this macro, you can send an ID that is of UUID type and call it if there is a record in the database.\nThe output of this macro is map.\n\n\n# Example\n```elixir\ncrud_get_record(\"6d80d5f4-781b-4fa8-9796-1821804de6ba\")\n```\n\nOutputs:\n\n```\n{:error, :not_found, String.t()} | struct()\n```","ref":"MishkaDeveloperTools.DB.CRUD.html#crud_get_record/1-macro-finding-a-record-in-a-database-with-the-help-of-id"},{"type":"callback","title":"MishkaDeveloperTools.DB.CRUD.delete/1","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#c:delete/1"},{"type":"callback","title":"MishkaDeveloperTools.DB.CRUD.delete/2","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#c:delete/2"},{"type":"callback","title":"MishkaDeveloperTools.DB.CRUD.edit/1","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#c:edit/1"},{"type":"callback","title":"MishkaDeveloperTools.DB.CRUD.show_by_field/2","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#c:show_by_field/2"},{"type":"callback","title":"MishkaDeveloperTools.DB.CRUD.show_by_id/1","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#c:show_by_id/1"},{"type":"type","title":"MishkaDeveloperTools.DB.CRUD.record_input/0","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#t:record_input/0"},{"type":"type","title":"MishkaDeveloperTools.DB.CRUD.repo_data/0","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#t:repo_data/0"},{"type":"type","title":"MishkaDeveloperTools.DB.CRUD.repo_error/0","doc":"","ref":"MishkaDeveloperTools.DB.CRUD.html#t:repo_error/0"},{"type":"module","title":"MishkaDeveloperTools.Helper.Derive","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.html"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.derive/2","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.html#derive/2"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.derive/3","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.html#derive/3"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.error_handler/2","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.html#error_handler/2"},{"type":"module","title":"MishkaDeveloperTools.Helper.Derive.Parser","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.Parser.html"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.Parser.convert_to_atom_map/1","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.Parser.html#convert_to_atom_map/1"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.Parser.parser/1","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.Parser.html#parser/1"},{"type":"module","title":"MishkaDeveloperTools.Helper.Derive.SanitizerDerive","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.SanitizerDerive.html"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.SanitizerDerive.call/2","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.SanitizerDerive.html#call/2"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.SanitizerDerive.sanitize/2","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.SanitizerDerive.html#sanitize/2"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.SanitizerDerive.validate_pattern/3","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.SanitizerDerive.html#validate_pattern/3"},{"type":"module","title":"MishkaDeveloperTools.Helper.Derive.ValidationDerive","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.ValidationDerive.html"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.ValidationDerive.call/2","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.ValidationDerive.html#call/2"},{"type":"function","title":"MishkaDeveloperTools.Helper.Derive.ValidationDerive.validate/3","doc":"","ref":"MishkaDeveloperTools.Helper.Derive.ValidationDerive.html#validate/3"},{"type":"extras","title":"Mishka Elixir Developer Tools","doc":"# Mishka Elixir Developer Tools\n\nWe tried to deliver a series of our client's [**CMS**](https://github.com/mishka-group/mishka-cms) built on [**Elixir**](https://elixir-lang.org/) at the start of the [**Mishka Group**](https://github.com/mishka-group) project, but we recently archived this open-source project and have yet to make plans to rework and expand it. This system was created using [**Phoenix**](https://www.phoenixframework.org/) and [**Phoenix LiveView**](https://hexdocs.pm/phoenix_live_view/Phoenix.LiveView.html). After a long period, a series of macros and functional modules emerged from this project and our other projects, which we are gradually publishing in this library.\n\n---\n\n- #### [GuardedStruct](https://github.com/mishka-group/mishka_developer_tools/blob/master/guidance/guarded-struct.md)\n\n- #### [Basic CRUD](https://github.com/mishka-group/mishka_developer_tools/blob/master/guidance/crud.md)\n\n---\n\n> Mishka developer tools provides some macros and modules to make creating your elixir application as easy as possible","ref":"readme.html"},{"type":"extras","title":"Installation - Mishka Elixir Developer Tools","doc":"The package can be installed by adding `mishka_developer_tools` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:mishka_developer_tools, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThe docs can be found at [https://hexdocs.pm/mishka_developer_tools](https://hexdocs.pm/mishka_developer_tools).","ref":"readme.html#installation"}],"content_type":"text/markdown"}